function f_data_driven_model(varargin)
disp(' ');
disp('========== DATA DRIVEN MODEL ==========');
A=varargin{1};
x=varargin{2};
y=varargin{3};
z=varargin{4};
time=varargin{5};
forcing=varargin{6};

%- following are default value for some argument
close all;
with_seasonal=1;
percent_training=75;
mode_to_explain = 9; 
run_id='test';
avg_window=30;
ntry=100; % number of nn tries
use_anomaly=0; % 1 to use anomaly to select forcing; 
som=2; % 1 for single linear; 2 for multiple linear regression
%- additional options 
for iopt = 7 : 2 : nargin
	optvalue = varargin{iopt+1};
	switch lower(varargin{iopt})
        case 'avg_window'
            avg_window=optvalue;
        case 'with_seasonal'
            with_seasonal=optvalue;
        case 'percent_training'
            percent_training=optvalue;
        case 'use_anomaly'
            use_anomaly=optvalue;
        case 'run_id'
            run_id=optvalue;
        case 'ntry'
            ntry=optvalue;
        case 'mode_to_explain'
            mode_to_explain=optvalue;
        case 'som'
            som=optvalue;
        otherwise
            error(['argument not recoganized (due to spelling error mostly): ' varargin{iopt}]);
    end
end

%% create some directories
if ~exist('mat','dir')
    mkdir('mat');
end
if ~exist('figs','dir')
    mkdir('figs');
end
if ~exist(['mat/' run_id],'dir')
    mkdir(['mat/' run_id]);
end
if ~exist(['figs/' run_id],'dir')
    mkdir(['figs/' run_id]);
end
%% seperate the data into 75% for training and eof, and another 25% for testing; 
rn=round(percent_training/100*length(time));
training.A=A(:,1:rn);
training.time=time(1:rn);

testing.A=A(:,rn+1:end);
testing.time=time(rn+1:end);

%% eof analysis first, using the training data set only
eof=f_eof_svd(training.A,training.time,y,z); 

%% train each mode, used the 75% of records only, save the training model
target.time=training.time; 
for imode=1:mode_to_explain
    target.var=eof.ctime(imode,:);
    pname=['mode' num2str(imode,'%2.2d')];
    out_mat=['mat/' run_id '/' pname '_nn.mat'];
    if exist(out_mat,'file')
        disp(['Already exist: ' out_mat ]);
        continue;
    end
    f_train_each_mode(forcing,target,'pname',pname,'out_mat',out_mat,...
        'run_id',run_id,'ntry',ntry,'use_anomaly',use_anomaly,'with_seasonal',with_seasonal,...
        'som',som);
end

%% testing the model for each mode, using last 25% of records, and calculate statistic index for the model performance
for imode=1:mode_to_explain
    pname=['mode' num2str(imode,'%2.2d')];
    load(['mat/' run_id '/' pname '_nn.mat']);
    disp(['Testing based on NN from ' 'mat/' run_id '/' pname '_nn.mat']);
    test_forcing=f_get_test_forcing(forcing,asft,testing.time,'use_anomaly',use_anomaly);

    if with_seasonal==1&&use_anomaly==1
        x1=f_get_seasonal(training.time,eof.ctime(imode,:),testing.time);
        test_forcing.X=[x1; test_forcing.X];
    end
    
    for itry=1:length(nntrain)
        yp(itry,:) = nntrain(itry).nn.net(test_forcing.X,'useParallel','no');
    end
    yp(yp>1)=1; 
    yp(yp<-1)=-1;
    ypo=yp;
    
    if with_seasonal==1
        testing.mode_pred(imode,:)=mean(yp,1);
    else
        yseasonal=f_get_seasonal(training.time,eof.ctime(imode,:),testing.time);
        testing.mode_pred(imode,:)=mean(yp,1)+yseasonal;
    end
    testing.mode_pred_se(imode,:)=std(yp)/sqrt(length(nntrain));
    testing.mode_pred_std(imode,:)=std(yp);
    
    for itry=1:length(nntrain)
        if imode==1
            testing.AP2(itry,:,:)=repmat(eof.ltm',[1,length(testing.time)]);
        end
        testing.AP2(itry,:,:)=testing.AP2(itry,:,:)+...
            reshape(eof.u(imode,:)'*ypo(itry,:),1,length(x),length(testing.time));
    end
end

%% get uncertainties
testing.AP_std2=nanstd(testing.AP2,1);
testing.AP_se2=testing.AP_std2/sqrt(length(nntrain));
testing.AP2=nanmean(testing.AP2,1); %AP from 3D to 2D
testing.AP2=squeeze(testing.AP2);
testing.AP_se2=squeeze(testing.AP_se2);
testing.AP_std2=squeeze(testing.AP_std2);

%% compare the real and predicted value in the testing dataset
figure('position',[10 10 1200 600],'color','w');
ip=0;
for ist=20:20:320
    ip=ip+1;
    subplot(4,4,ip); hold on;
    plot(testing.time,testing.A(ist,:),'ko','markerfacecolor','k','markersize',3);
    plot(testing.time,testing.AP2(ist,:),'rs-','markerfacecolor','r','markersize',3);
    f_fill_err(testing.time,testing.AP2(ist,:),testing.AP_std2(ist,:),'r'); alpha(0.2);
    xlim([min(testing.time),max(testing.time)]);
    datetick('x','yyyy','keeplimits');
    ylim([0,15]);
    box on;
    f_add_pname(0.05,0.95,['igrid:',num2str(ist)]);
end
save model forcing testing training A x y z time eof
movefile('model.mat',['mat/' run_id '/'],'f');
disp(['Completed! results save into mat/' run_id '/model.mat']);
