function out=f_eof_svd(varargin)
disp('## f_eof_svd: perform EOF analysis for matrix A using svd method');
A=varargin{1};
time=varargin{2};
x=varargin{3};
y=varargin{4};
rm_ltm=1; %by defalt, it wiwll remove the long term mean
land=[];
percent=100; % 100 means all data will be used; otherwise, only partial dataset is used.  
out_mat='';
for iopt = 5 : 2 : nargin
	optvalue = varargin{iopt+1};
	switch lower(varargin{iopt})
        case 'rm_ltm'
            rm_ltm=optvalue;
        case 'yn'
            yn=optvalue;
        case 'clims'
            clims=optvalue;
        case 'mark_station'
            mark_station=optvalue;
        case 'percent' 
            percent=optvalue; 
            if percent>100||percent<1
                error('in f_eof_svd: percent should be in [1 100]')
            end
        case 'out_mat'
            out_mat=optvalue;
        case 'land'
            land=optvalue; %a structure with x and y
        otherwise
            error(['argument ' varargin{iopt} 'not needed']);
    end
end

if size(A,1)~=length(x) 
    A=A';
end

if percent ~= 100
    disp(['in f_eof_svd: the first ' num2str(percent) '% of records are used']);
    rn=1:round(length(time)*percent/100);
    time=time(rn);
    A=A(:,rn);
end

ttime=datevec(time);
for ist=1:size(A,1)
    for imon=1:12
        seasonal(imon)=nanmean(A(ist,ttime(:,2)==imon));
    end
    a=find(isnan(A(ist,:)));
    if isempty(a)
        continue;
    end
    for i=reshape(a,1,[])
        tmptime=datevec(time(i));
        imon=tmptime(2);
        A(ist,i)=seasonal(imon);
    end
end

disp('f_eof_svd: Calculate long term mean');
for ist=1:size(A,1)
    ltm(ist)=nanmean(A(ist,:));
    if rm_ltm==1
        if ist==1
            disp('f_eof_svd: Remove long term mean');
        end
        A(ist,:)=A(ist,:)-ltm(ist);
    end
end

[u,s,v]=svd(A);

ctime=s*v';

gd_svd=diag(s);
sigma=gd_svd.^2/size(A,2);
percent_explained=sigma./sum(sigma);

for cn=1:size(ctime,1)
    factor=max(abs(ctime(cn,:)));
    if max(ctime(cn,:))~=max(abs(ctime(cn,:)))
        ctime(cn,:)=-1*ctime(cn,:);
        u(:,cn)=u(:,cn).*-1;
    end
    ctime(cn,:)=ctime(cn,:)./factor;
    u(:,cn)=u(:,cn).*factor;
end
u=u'; % make it to (imode,istation)
%%
disp('Percentage of each mode:');
disp('------------------------');
for i=1:10
    disp(['mode ' num2str(i),' : ' num2str(percent_explained(i)*100,'%.2f') '%']);
end
%%
if ~isempty(out_mat)
    save eof u time x y percent ctime A ltm percent_explained
    movefile('eof.mat',out_mat,'f');
end
out.u=u;
out.time=time;
out.x=x;
out.y=y;
out.percent_record=percent;
out.ctime=ctime;
out.A=A;
out.ltm=ltm;
out.percent_var_explain=percent_explained;
%% plotting the map for the first 4 modes
for imode=1:4
    figure('position',[10 10 800,300]); colormap(flipud(f_colormap_RdBu_24()));
    subplot(1,2,1); hold on;
    scatter(x,y,300,u(imode,:),'.');
    if ~isempty(land) % if provided land, contour plot will be shown
        factor=(max(y)-min(y))/(max(x)-min(x));
        x=x*factor; % this is used to make the y and x in same scale, avoiding the mismatch
        [X,Y]=meshgrid(min(x):(max(x)-min(x))/99:max(x),min(y):(max(y)-min(y))/99:max(y));
        Z=griddata(x,y,u(imode,:),X,Y,'cubic');
        x=x/factor; X=X/factor;
        pcolor(X,Y,Z); shading interp;
        fill(land.x,land.y,[1 1 1]*0.5);
    end
    xlim([min(x),max(x)]);
    ylim([min(y),max(y)]);
    box on; grid on;
    set(gca,'tickdir','out');
    f_add_pname(0.05,0.95,['mode #' num2str(imode)]);
    
    subplot(1,2,2);
    f_plotSeasonal5(time,ctime(imode,:)); % show the seasonality
    f_add_pname(0.05,0.95,['mode #' num2str(imode)]);
end

