function nn=f_neural(varargin)
X=varargin{1};
Y=varargin{2};
layers=varargin{3};

% additional options and their default values 
max_epochs=100; 
method='trainlm';
trainRatio=0.8;
valRatio=0.2;
for iopt = 4 : 2 : nargin
	optvalue = varargin{iopt+1};
	switch lower(varargin{iopt})
        case 'max_epochs'
            max_epochs=optvalue;
        case 'method'
            method=optvalue;
        case 'trainratio'
            trainRatio=optvalue;
        case 'valratio'
            valRatio=optvalue;
        otherwise
            disp(['argument not recoganized : ' varagin{iopt}]);
    end
end

% check the X and Y dimension
if size(Y,1)==1||size(Y,2)==1
    Y=reshape(Y,1,[]);
else
    error('Y must be a column vector, or row vector');
end
if size(X,2)~=length(Y)
    X=X';
end

if size(X,2)~=length(Y)
    error('mismatched X and Y; column number of X and Y should be equal');
end
if ~isempty(find(isnan(X)))
    error('NaN found in X');
end
if ~isempty(find(isnan(Y)))
    error('NaN found in Y');
end

% perform the neural network
net = fitnet(layers,method);   % set layers and neurals
net.trainParam.epochs = max_epochs;  % set max epochs
net.divideParam.trainRatio=trainRatio;
net.divideParam.valRatio=valRatio;
net.divideParam.testRatio=1-valRatio-trainRatio;
[net,tr] = train(net,X,Y,'useParallel','no','showResources','no');
yp = net(X,'useParallel','no');          % y prediction (fitting results)
errors = gsubtract(Y,yp);           % errors    
performance = perform(net,Y,yp);    % performace

% function output
nn.net=net;  % the trained model; 
nn.errors=errors;
nn.performance=performance;
nn.layers=layers;
nn.X=X;
nn.Y=Y;
nn.yp=yp;
nn.tr=tr;